#include "Compiler.h"
#include "Vm.h"

namespace JV {

std::vector<uint8_t> Compiler::CompileExample() {
    std::vector<uint8_t> code;

    // n = 10
    code.push_back(OP_CONST);
    code.push_back(10);
    code.push_back(OP_STORE);  // R4 = n
    code.push_back(4);

    // alloc heap[n]
    code.push_back(OP_ALLOC);  // heap = malloc(n)
    code.push_back(10);
    code.push_back(OP_STORE);  // R5 = heap base address
    code.push_back(5);

    // i = 2
    code.push_back(OP_CONST);
    code.push_back(2);
    code.push_back(OP_STORE);  // R0 = i
    code.push_back(0);

    // a = 0
    code.push_back(OP_CONST);
    code.push_back(0);
    code.push_back(OP_STORE);  // R1 = a
    code.push_back(1);

    // b = 1
    code.push_back(OP_CONST);
    code.push_back(1);
    code.push_back(OP_STORE);  // R2 = b
    code.push_back(2);

    // store fib(0) at heap[0]
    code.push_back(OP_LOAD);   // R5 (heap base)
    code.push_back(5);
    code.push_back(OP_CONST);  // push 0
    code.push_back(0);
    code.push_back(OP_ADD);    // heap + 0
    code.push_back(OP_CONST);  // value: 0
    code.push_back(0);
    code.push_back(OP_STORE_MEM);

    // print fib(0)
    code.push_back(OP_CONST);  // print 0
    code.push_back(0);
    code.push_back(OP_PRINT);

    // store fib(1) at heap[1]
    code.push_back(OP_LOAD);   // R5
    code.push_back(5);
    code.push_back(OP_CONST);  // +1
    code.push_back(1);
    code.push_back(OP_ADD);
    code.push_back(OP_CONST);  // value: 1
    code.push_back(1);
    code.push_back(OP_STORE_MEM);

    // print fib(1)
    code.push_back(OP_CONST);
    code.push_back(1);
    code.push_back(OP_PRINT);

    // === loop:
    int loop_start = code.size();

    // if i >= n: halt
    code.push_back(OP_LOAD);  // i
    code.push_back(0);
    code.push_back(OP_LOAD);  // n
    code.push_back(4);
    code.push_back(OP_SUB);
    code.push_back(OP_CONST); // if i - n >= 0 → skip CALL
    code.push_back(0);
    code.push_back(OP_SUB);   // dummy conditional
    // (TODO: implement real conditional branching)

    // fib = a + b
    code.push_back(OP_LOAD);  // a
    code.push_back(1);
    code.push_back(OP_LOAD);  // b
    code.push_back(2);
    code.push_back(OP_ADD);
    code.push_back(OP_STORE); // R3 = fib
    code.push_back(3);

    // store fib(i) in heap[i]
    code.push_back(OP_LOAD);  // R5 (heap base)
    code.push_back(5);
    code.push_back(OP_LOAD);  // R0 (i)
    code.push_back(0);
    code.push_back(OP_ADD);   // addr = base + i
    code.push_back(OP_LOAD);  // fib value (R3)
    code.push_back(3);
    code.push_back(OP_STORE_MEM);

    // print fib(i)
    code.push_back(OP_LOAD);
    code.push_back(3);
    code.push_back(OP_PRINT);

    // a = b
    code.push_back(OP_LOAD);
    code.push_back(2);
    code.push_back(OP_STORE);
    code.push_back(1);

    // b = fib
    code.push_back(OP_LOAD);
    code.push_back(3);
    code.push_back(OP_STORE);
    code.push_back(2);

    // i = i + 1
    code.push_back(OP_LOAD);
    code.push_back(0);
    code.push_back(OP_CONST);
    code.push_back(1);
    code.push_back(OP_ADD);
    code.push_back(OP_STORE);
    code.push_back(0);

    // jump back to loop
    code.push_back(OP_CALL);
    code.push_back(loop_start);

    code.push_back(OP_HALT);
    return code;
}

} // namespace JV
