#ifndef VM_H
#define VM_H

#include <vector>
#include <stack>
#include <cstdint>
#include <array>

namespace JV {

    constexpr int MAX_REGISTERS = 8;
    constexpr int STACK_SIZE = 1024;
    static constexpr int HEAP_SIZE = 256; // "size_t" give wrnings




    enum OpCode : uint8_t {
        OP_HALT = 0,
        OP_CONST,
        OP_STORE,
        OP_LOAD,
        OP_ADD,
        OP_SUB,
        OP_MUL,
        OP_DIV,
        OP_PRINT,
        OP_CALL,
        OP_RET,
        OP_ALLOC,      // Allocate N cells, return address
        OP_STORE_MEM,  // Stack: [value, addr]
        OP_LOAD_MEM,    // Stack: [addr]
        OP_DEBUG
    };

    class VM {
    public:
        explicit VM(const std::vector<uint8_t>& bytecode);
        void Run();

    private:
        std::vector<uint8_t> m_Bytecode;
        std::stack<int> m_Stack;
        std::vector<int> m_Heap;
        size_t m_HeapPtr = 0;
        std::array<int, MAX_REGISTERS> m_Registers{};
        std::stack<size_t> m_CallStack;
        size_t m_IP = 0;

        int Pop();
        void Push(int val);
        void Debug();

        void Alloc();
        void StoreMem();
        void LoadMem();



    };

} // namespace JV

#endif // VM_H
